from typing import Union

from textual.app import ComposeResult
from textual.containers import Vertical, VerticalScroll
from textual.screen import ModalScreen
from textual.widgets import Static


class ErrorModal(ModalScreen):
    def __init__(
        self,
        title: str,
        header: str,
        error: BaseException,
        name: Union[str, None] = None,
        id: Union[str, None] = None,
        classes: Union[str, None] = None,
    ) -> None:
        self.title = title
        self.header = header
        self.error = error
        super().__init__(name, id, classes)

    def compose(self) -> ComposeResult:
        with Vertical(id="outer"):
            yield Static(self.header, id="error_header")
            with Vertical(id="inner"):
                with VerticalScroll():
                    yield Static(str(self.error), id="error_info")
            yield Static("Press any key to continue.", id="error_footer")

    def on_mount(self) -> None:
        container = self.query_one("#outer")
        container.border_title = self.title

    def on_key(self) -> None:
        self.app.pop_screen()



from pathlib import Path

import click

from harlequin import Harlequin


@click.command()
@click.version_option(package_name="harlequin")
@click.argument(
    "db_path",
    default=":memory:",
    nargs=1,
    type=click.Path(path_type=Path),
)
def harlequin(db_path: Path) -> None:
    tui = Harlequin(db_path=db_path)
    tui.run()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['harlequin', 'harlequin.tui', 'harlequin.tui.components']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'duckdb>=0.7.1,<0.8.0',
 'shandy-sqlfmt>=0.18.0,<0.19.0',
 'textual>=0.22.3,<0.25.0']

entry_points = \
{'console_scripts': ['harlequin = harlequin.cli:harlequin']}

setup_kwargs = {
    'name': 'harlequin',
    'version': '0.0.8',
    'description': 'A Text User Interface for DuckDB',
    'long_description': '# harlequin\nA Terminal-based SQL IDE for DuckDB.\n\n![harlequin TUI](harlequinv004.gif)\n\n(A Harlequin is also a [pretty duck](https://en.wikipedia.org/wiki/Harlequin_duck).)\n\n![harlequin duck](harlequin.jpg)\n\n## Installing Harlequin\n\nAfter installing Python 3.8 or above, install Harlequin using `pip` or `pipx` with:\n\n```bash\npipx install harlequin\n```\n\n> **Tip:**\n>\n> You can run invoke directly with [`pipx run`](https://pypa.github.io/pipx/examples/#pipx-run-examples) anywhere that `pipx` is installed. For example:\n> - `pipx run harlequin --help`\n> - `pipx run harlequin ./my.duckdb`\n\n\n## Running Harlequin in a Container\n\nWithout a database file:\n\n```bash\ndocker run ghcr.io/tconbeer/harlequin:latest\n```\n\nMounting a database file `./foo.db` into the container\'s working directory, `/data`:\n\n```bash\ndocker run -v $(pwd)/foo.db:/data/bar.db ghcr.io/tconbeer/harlequin:latest harlequin bar.db\n```\n\n## Using Harlequin\n\nFrom any shell, to open a DuckDB database file:\n\n```bash\nharlequin "path/to/duck.db"\n```\n\nTo open an in-memory DuckDB session, run Harlequin with no arguments:\n\n```bash\nharlequin\n```\n\n### Viewing the Schema of your Database\n\nWhen Harlequin is open, you can view the schema of your DuckDB database in the left sidebar. You can use your mouse or the arrow keys + enter to navigate the tree. The tree shows schemas, tables/views and their types, and columns and their types.\n\n### Editing a Query\n\nThe main query editor is a full-featured text editor, with features including syntax highlighting, auto-formatting with ``ctrl + ` ``, text selection, copy/paste, and more.\n\nYou can save the query currently in the editor with `ctrl + s`. You can open a query in any text or .sql file with `ctrl + o`.\n\n### Running a Query and Viewing Results\n\nTo run a query, press `ctrl + enter`. Up to 50k records will be loaded into the results pane below the query editor. When the focus is on the data pane, you can use your arrow keys or mouse to select different cells.\n\n### Exiting Harlequin\n\nPress `ctrl + q` to quit and return to your shell.',
    'author': 'Ted Conbeer',
    'author_email': 'tconbeer@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Union

from textual.app import ComposeResult
from textual.widgets import ContentSwitcher, DataTable, LoadingIndicator


class ResultsTable(DataTable):
    DEFAULT_CSS = """
        ResultsTable {
            height: 100%;
            width: 100%;
        }
    """


class ResultsViewer(ContentSwitcher):
    TABLE_ID = "data"
    LOADING_ID = "loading"

    def compose(self) -> ComposeResult:
        yield ResultsTable(id=self.TABLE_ID)
        yield LoadingIndicator(id=self.LOADING_ID)

    def on_mount(self) -> None:
        self.border_title = "Query Results"
        self.current = self.TABLE_ID

    def set_not_responsive(
        self, max_rows: Union[int, None] = None, total_rows: Union[int, None] = None
    ) -> None:
        if (total_rows and not max_rows) or (
            total_rows and max_rows and total_rows <= max_rows
        ):
            self.border_title = f"LOADING {total_rows:,} Records."
        elif total_rows and max_rows:
            self.border_title = f"LOADING {max_rows:,} of {total_rows:,} Records."
        else:
            self.border_title = "Running Query"
        self.add_class("non-responsive")

    def increment_progress_bar(self) -> None:
        self.border_title = f"{self.border_title}."

    def set_responsive(
        self,
        max_rows: Union[int, None] = None,
        total_rows: Union[int, None] = None,
        did_run: bool = True,
    ) -> None:
        if (total_rows and not max_rows) or (
            total_rows and max_rows and total_rows <= max_rows
        ):
            self.border_title = f"Query Results ({total_rows:,} Records)"
        elif total_rows and max_rows:
            self.border_title = (
                f"Query Results (Showing {max_rows:,} of {total_rows:,} Records)."
            )
        elif not did_run:
            self.border_title = "Query Results"
        else:
            self.border_title = "Query Returned No Records"
        self.remove_class("non-responsive")

    def show_table(self) -> None:
        self.current = self.TABLE_ID

    def get_table(self) -> ResultsTable:
        table = self.get_child_by_id(self.TABLE_ID)
        assert isinstance(table, ResultsTable)
        return table

    def show_loading(self) -> None:
        self.current = self.LOADING_ID

    def get_loading(self) -> LoadingIndicator:
        loading = self.get_child_by_id(self.LOADING_ID)
        assert isinstance(loading, LoadingIndicator)
        return loading



COLUMN_TYPE_MAPPING = {
    "BIGINT": "##",
    "BIT": "010",
    "BOOLEAN": "t/f",
    "BLOB": "0b",
    "DATE": "d",
    "DOUBLE": "#.#",
    "DECIMAL": "#.#",
    "HUGEINT": "###",
    "INTEGER": "#",
    "INTERVAL": "|-|",
    "REAL": "#.#",
    "SMALLINT": "#",
    "TIME": "t",
    "TIMESTAMP": "ts",
    "TIMESTAMP WITH TIME ZONE": "ttz",
    "TINYINT": "#",
    "UBIGINT": "u##",
    "UINTEGER": "u#",
    "USMALLINT": "u#",
    "UTINYINT": "u#",
    "UUID": "uid",
    "VARCHAR": "s",
    "LIST": "[]",
    "STRUCT": "{}",
    "MAP": "{}",
}


def short_type(native_type: str) -> str:
    return COLUMN_TYPE_MAPPING.get(native_type.split("(")[0], "?")



from textual.app import ComposeResult
from textual.containers import Vertical
from textual.screen import ModalScreen
from textual.widgets import Input, Label


class FilenameModal(ModalScreen):
    BINDINGS = [("escape", "cancel", "Cancel")]

    def compose(self) -> ComposeResult:
        with Vertical(id="file_container"):
            yield Label("Enter a path to the file:", id="file_header")
            yield Input(id=self.id)
            yield Label("Or press Esc to cancel.", id="file_footer")

    def on_mount(self) -> None:
        input = self.query_one(Input)
        self.set_focus(input)

    def action_cancel(self) -> None:
        self.app.pop_screen()
